package com.fs.course.service.impl;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fs.course.mapper.FsUserCoursePeriodDaysMapper;
import com.fs.store.mapper.FsUserMapper;
import com.google.common.collect.Lists;
import lombok.AllArgsConstructor;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.course.mapper.FsUserWatchCourseStatisticsMapper;
import com.fs.course.domain.FsUserWatchCourseStatistics;
import com.fs.course.service.IFsUserWatchCourseStatisticsService;


/**
 * 会员看课统计-按课程统计Service业务层处理
 *
 * @author fs
 * @date 2025-06-17
 */
@Service
@AllArgsConstructor
public class FsUserWatchCourseStatisticsServiceImpl extends ServiceImpl<FsUserWatchCourseStatisticsMapper, FsUserWatchCourseStatistics> implements IFsUserWatchCourseStatisticsService {

    private FsUserCoursePeriodDaysMapper fsUserCoursePeriodDaysMapper;

    private FsUserMapper fsUserMapper;

    @Autowired
    private SqlSessionFactory sqlSessionFactory;

    /**
     * 查询会员看课统计-按课程统计
     *
     * @param id 会员看课统计-按课程统计主键
     * @return 会员看课统计-按课程统计
     */
    @Override
    public FsUserWatchCourseStatistics selectFsUserWatchCourseStatisticsById(Long id)
    {
        return baseMapper.selectFsUserWatchCourseStatisticsById(id);
    }

    /**
     * 查询会员看课统计-按课程统计列表
     *
     * @param fsUserWatchCourseStatistics 会员看课统计-按课程统计
     * @return 会员看课统计-按课程统计
     */
    @Override
    public List<FsUserWatchCourseStatistics> selectFsUserWatchCourseStatisticsList(FsUserWatchCourseStatistics fsUserWatchCourseStatistics)
    {
        return baseMapper.selectFsUserWatchCourseStatisticsList(fsUserWatchCourseStatistics);
    }

    /**
     * 新增会员看课统计-按课程统计
     *
     * @param fsUserWatchCourseStatistics 会员看课统计-按课程统计
     * @return 结果
     */
    @Override
    public int insertFsUserWatchCourseStatistics(FsUserWatchCourseStatistics fsUserWatchCourseStatistics)
    {
        return baseMapper.insertFsUserWatchCourseStatistics(fsUserWatchCourseStatistics);
    }

    /**
     * 修改会员看课统计-按课程统计
     *
     * @param fsUserWatchCourseStatistics 会员看课统计-按课程统计
     * @return 结果
     */
    @Override
    public int updateFsUserWatchCourseStatistics(FsUserWatchCourseStatistics fsUserWatchCourseStatistics)
    {
        return baseMapper.updateFsUserWatchCourseStatistics(fsUserWatchCourseStatistics);
    }

    /**
     * 批量删除会员看课统计-按课程统计
     *
     * @param ids 需要删除的会员看课统计-按课程统计主键
     * @return 结果
     */
    @Override
    public int deleteFsUserWatchCourseStatisticsByIds(Long[] ids)
    {
        return baseMapper.deleteFsUserWatchCourseStatisticsByIds(ids);
    }

    /**
     * 删除会员看课统计-按课程统计信息
     *
     * @param id 会员看课统计-按课程统计主键
     * @return 结果
     */
    @Override
    public int deleteFsUserWatchCourseStatisticsById(Long id)
    {
        return baseMapper.deleteFsUserWatchCourseStatisticsById(id);
    }

    @Override
    public void insertWatchCourseStatistics() {
        // 1、获取统计数据
        // 查询课程相关数据
        List<FsUserWatchCourseStatistics> fsUserWatchCourseStatistics = fsUserCoursePeriodDaysMapper.selectDaysCountList();

        // 查询统计数据
        List<FsUserWatchCourseStatistics> watchLog = fsUserMapper.selectWatchLogCount();
        List<FsUserWatchCourseStatistics> redPacketLog = fsUserMapper.selectRedPacketLogCount();
        List<FsUserWatchCourseStatistics> answerLog = fsUserMapper.selectAnswerLogCount();
        List<FsUserWatchCourseStatistics> userTotal = fsUserMapper.selectFsUserDetail();

        // 转化为自定义键的map
        Map<String, FsUserWatchCourseStatistics> watchLogMap = watchLog.stream().collect(Collectors.toMap(k -> String.format("%s-%s-%s", k.getPeriodId(), k.getVideoId(), k.getCompanyUserId()), v -> v));
        Map<String, FsUserWatchCourseStatistics> redPacketLogMap = redPacketLog.stream().collect(Collectors.toMap(k -> String.format("%s-%s-%s", k.getPeriodId(), k.getVideoId(), k.getCompanyUserId()), v -> v));
        Map<String, FsUserWatchCourseStatistics> answerLogMap = answerLog.stream().collect(Collectors.toMap(k -> String.format("%s-%s-%s", k.getPeriodId(), k.getVideoId(), k.getCompanyUserId()), v -> v));
        Map<Long, FsUserWatchCourseStatistics> userTotalMap = userTotal.stream()
                .collect(Collectors.toMap(
                        FsUserWatchCourseStatistics::getCompanyUserId,
                        Function.identity()
                ));


        // 处理数据
        List<FsUserWatchCourseStatistics> list = new ArrayList<>();
        for (FsUserWatchCourseStatistics data : fsUserWatchCourseStatistics) {
            FsUserWatchCourseStatistics vo = new FsUserWatchCourseStatistics();
            String key = String.format("%s-%s-%s", data.getPeriodId(), data.getVideoId(), data.getCompanyUserId());
            FsUserWatchCourseStatistics watchLogData = watchLogMap.get(key);
            FsUserWatchCourseStatistics redPacketLogData = redPacketLogMap.get(key);
            FsUserWatchCourseStatistics answerLogData = answerLogMap.get(key);
            FsUserWatchCourseStatistics userTotalData = userTotalMap.get(data.getCompanyUserId());
            BeanUtils.copyProperties(data, vo);
            if(userTotalData != null){
                vo.setUserNum(userTotalData.getUserNum());
                vo.setNewUserNum(userTotalData.getNewUserNum());
            }

            if(watchLogData != null) {
                vo.setWatchNum(watchLogData.getWatchNum());
                vo.setCompleteWatchNum(watchLogData.getCompleteWatchNum());
                vo.setCompleteWatchRate(watchLogData.getCompleteWatchRate());
            }

            if(redPacketLogData != null) {
                vo.setRedPacketNum(redPacketLogData.getRedPacketNum());
                vo.setRedPacketAmount(redPacketLogData.getRedPacketAmount());
            }

            if(answerLogData != null) {
                vo.setAnswerNum(answerLogData.getAnswerNum());
                vo.setAnswerRightNum(answerLogData.getAnswerRightNum());
                vo.setAnswerRightRate(answerLogData.getAnswerRightRate());
            }

            vo.setCreateTime(new Date());
            vo.setUpdateTime(new Date());
            list.add(vo);
        }

        //2、分批次插入数据
        this.batchInsert(list);

    }

    private void batchInsert(List<FsUserWatchCourseStatistics> list) {
        // 分批次处理，一次提交500条
        List<List<FsUserWatchCourseStatistics>> batches = Lists.partition(list, 500);
        batches.forEach(batch -> {
            SqlSession sqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH);
            try {
                FsUserWatchCourseStatisticsMapper mapper = sqlSession.getMapper(FsUserWatchCourseStatisticsMapper.class);
                batch.forEach(mapper::insertFsUserWatchCourseStatisticsTask);
                sqlSession.commit();
            } finally {
                sqlSession.close();
            }
        });
    }

}
